/**
 * Classe de calcul du zero d'une fonction par diverses methodes :
 *
 *  -Elargissement de l'intervalle
 *  -Dichotomie
 *  -Algorithme de Newton-Raphson
 *
 *
 * Parametres :
 *  @param rapportDeMasse Le rapport de masse des deux corps pesants
 *  @param intervalle Les bornes de l'intervalle de depart pour la dichotomie
 *  @param precision La taille de l'intervalle final souhaitee
 *  @param aMinimiser La fonction dont on recherche le zero
 *  @param deriveeDeMini La derivee de la fonction
 *
 *  @author Benoit CARRY
 *  @version 05/12/2005
 */
public class DetermineZero {


   /**
    * Constructeur de la classe DetermineZero
    *
    */
  public DetermineZero() {}



    /**
     * Methode de determination du zero d'une fonction a partir de celle-ci
     * Algorithme de dichotomie
     *
     *  @param intervalle Intervalle initial comprenant le zero de la fonction 
     *  @param precision Precision souhaitee
     *  @param paramIn Tableau de parametres dont depand la fonction dont on cherche le zero
     *  @param aMinimiser Fonction dont on cherche le zero
     *
     *  @return L'intervalle contenant le zero de la fonction, a la precision souhaitee
     */
  public double[] dichotomie(double[] intervalle, double precision, double[] paramIn, ChoixDeFonction aMinimiser) {

   //-Intermediaires de calcul pour effectuer la dichotomie
    double x1, x2, xtest;              //Abscisses des bornes
    double y1, y2, ytest;              //Ordonnees associees
    double[] resultat=new double[2];   //Couple d'abscisse des bornes


   //-Mise en place des couples (x,y) en debut et fin d'intervalle
    x1=intervalle[0];
     y1=aMinimiser.fonction( x1, paramIn);
    x2=intervalle[1];
     y2=aMinimiser.fonction( x2, paramIn);


   //-Dichotomie jusqu'a la precision souhaitee
    while ( Math.abs(x2-x1) >= precision && y1*y2 < 0 ) {

     //-On calcule la valeur mediane
      xtest = 0.5*( x1+x2 );
      ytest = aMinimiser.fonction( xtest, paramIn);

     //-Reduction de l'intervalle a moitie
      if ( y1*ytest<0 ) {
        x2 = xtest;
        y2 = aMinimiser.fonction( x2, paramIn);
      }
      else {
        x1 = xtest;
        y1 = aMinimiser.fonction( x1, paramIn);
      }

    }
  
   //-Affectation des bornes a un vecteur (pour la sortie)
    resultat[0] = x1;
    resultat[1] = x2;

    return resultat;

  }







    /**
     * Methode de determination du zero d'une fonction a partir de celle-ci et de sa derivee
     * Algorithme de Newton Raphson
     *
     *  @param position Position initiale proche du zero suppose
     *  @param precision Precision souhaitee
     *  @param paramIn Tableau de parametres dont depandent la fonction dont on cherche le zero et sa derivee
     *  @param aMinimiser Fonction dont on cherche le zero
     *  @param deriveeDeMini Sa derivee
     *
     *  @return Rnvoie une approximation du zero de la fonction a la precision souhaitee
     */
  public double newtonRaphson(double position, double precision, double[] paramIn, ChoixDeFonction aMinimiser, ChoixDeFonction deriveeDeMini) {

   //-Intermedaires de calcul
    double x,y,dy;                //Position de test, ordonnee et pente associees
    double ecart=precision+1e-6;  //Deplacement de la position test entre 2 pas
    double coeffDeSousRelax=1;    //Limitation du NR pour eviter les cycles
    int nbIter=0;                 //Nombre d'iterations

   //-Debugging
    //System.out.println("Debut de NewtonRaphson");
   //-Debugging

   //-Positions initiales
    x = position;
    y = aMinimiser.fonction( x, paramIn);

   //-Boucle jusqu'a la precision souhaitee (en x et en y)
    while ( Math.abs(ecart) >= precision || Math.abs(y) >= precision ) {

     //-Valeur de la fonction et de sa derivee au point test
      y  = aMinimiser.fonction( x, paramIn);
      dy = deriveeDeMini.fonction( x, paramIn);

     //-Calcul de la nouvelle position test
      ecart = y/dy;
      x = x - coeffDeSousRelax*ecart;

      nbIter++;

      }

   //-Debugging
    //System.out.println("Nombre d'iterations : "+nbIter);
   //-Debugging

    return x;

  }







    /**
     * Methode de verification de la presence d'un zero dans l'intervalle
     *
     *  @param intervalle Intervalle initial comprenant le zero de la fonction 
     *  @param paramIn Tableau de parametres dont depand la fonction dont on cherche le zero
     *  @param aMinimiser Fonction dont on cherche le zero
     *
     *  @return Renvoie un intervalle contenant obligatoirement le zero de la fonction 
     */
  public double[] verifZero(double[] intervalle, double[] paramIn, ChoixDeFonction aMinimiser) {


   //-Intermediaires de calcul pour effectuer la verification
    double x1, x2;                      //Abscisses des bornes
    double y1, y2;                      //Ordonnees associees
    double[] resultat = new double[2];  //Couple d'abscisse des bornes
    double facteur=0.15;                //Facteur d'augmentation de l'intervalle


   //-Mise en place des couples (x,y) en debut et fin d'intervalle
    x1=intervalle[0];
     y1=aMinimiser.fonction( x1, paramIn);
    x2=intervalle[1];
     y2=aMinimiser.fonction( x2, paramIn);


   //-Debugging
    //if (y1*y2 <0) {
    //  System.out.println("Valeur dans l'intervalle propose...");
    //}
   //-Debugging


   //-Augmentation de la taille de l'intervalle jusqu'a contenir le zero
    while ( y1*y2 >= 0 ) {

     //-Augmente la taille de l'intervalle 
      x1 = x1 - (x2-x1)*facteur;
      x2 = x2 + (x2-x1)*facteur;
   
      y1=aMinimiser.fonction( x1, paramIn);
      y2=aMinimiser.fonction( x2, paramIn);

    }

  
   //Affectation des bornes a un vecteur (pour la sortie)
    resultat[0] = x1;
    resultat[1] = x2;


   //-Debugging
    //System.out.println("L'intervalle vaut : [ "+x1+" , "+x2+" ]");
   //-Debugging

    return resultat;

  }





}
